#ifndef MSP_CORE_THREAD_H_
#define MSP_CORE_THREAD_H_

namespace Msp {

/**
Base class for threads.  To create a thread for some task, derive it from this
class and implement the main() function.  Note that threads are not
automatically started upon creation - you must manually call launch() instead.
This is to allow initializing variables of the derived class before the thread
is started.
*/
class Thread
{
private:
	struct Private;

	Private *priv_;
	bool launched_;

protected:
	Thread();
private:
	Thread(const Thread &);
	Thread &operator=(const Thread &);
public:
	virtual ~Thread();

	void join();
	void kill();
protected:
	void launch();
	virtual void main() = 0;
};

} // namespace Msp

#endif
